==> org-lsst-ccs-subsystem-ocsbridge-2.2.3-sources.jar: org/lsst/ccs/subsystem/ocsbridge/OCSBridgeSALLayer.java <==
package org.lsst.ccs.subsystem.ocsbridge;

import java.time.Duration;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.sal.SAL;
import org.lsst.sal.camera.CameraCommand;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.CameraTelemetry;
import org.lsst.sal.SALException;
import org.lsst.sal.SALReceivedCommand;

/**
 *
 * @author tonyj
 */
public class OCSBridgeSALLayer extends OCSCommandExecutor {

    private final OCSBridge bridge;
    private final SAL<CameraCommand, CameraEvent, CameraTelemetry> mgr;
    private boolean shutdown = false;
    private static final Logger logger = Logger.getLogger(OCSBridgeSALLayer.class.getName());
    private Thread runThread;

    OCSBridgeSALLayer(OCSBridge bridge) {
        super(bridge);
        this.bridge = bridge;
        mgr = bridge.getConfig().getSALManager();
    }

    public void start() {

        Thread t = new Thread("SALCommandReceiver") {

            @Override
            public void run() {
                try {
                    runCommandLoop();
                } catch (SALException x) {
                    logger.log(Level.WARNING, "Failed to initialize SAL communication layer, "
                            + "check that SAL has been setup correctly. Reverting to standalone mode.", x);
                }
            }
        };
        t.start();
    }

    private void runCommandLoop() throws SALException {
        try {
            runThread = Thread.currentThread();

            while (!shutdown) {
                currentCommand = mgr.getNextCommand(Duration.ofMinutes(1));
                if (currentCommand == null) {
                    logger.fine("Still waiting for a command");
                } else {
                    bridge.execute(currentCommand.getCommand());
                }
            }
            //mgr.salShutdown();
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while waiting for commands", ex);
            throw ex;
        } finally {
            runThread = null;
        }
    }
    private SALReceivedCommand<CameraCommand> currentCommand;

    private void shutdown() throws InterruptedException {
        shutdown = true;
        runThread.join();
    }

    @Override
    protected void reportComplete(OCSCommandExecutor.OCSExecutor command) {
        super.reportComplete(command);
        try {
            currentCommand.reportComplete();
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    protected void reportError(OCSCommandExecutor.OCSExecutor command, Exception ex) {
        super.reportError(command, ex);
        try {
            currentCommand.reportError(ex);
        } catch (SALException x) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", x);
        }
    }

    @Override
    protected void acknowledgeCommand(OCSCommandExecutor.OCSExecutor command, Duration timeout) {
        super.acknowledgeCommand(command, timeout);
        try {
            currentCommand.acknowledgeCommand(timeout);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    protected void rejectCommand(OCSCommandExecutor.OCSExecutor command, String reason) {
        super.rejectCommand(command, reason);
        try {
            currentCommand.rejectCommand(reason, 0);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    void sendEvent(CameraEvent event) {
        super.sendEvent(event);
        try {
            mgr.logEvent(event);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error sending event to SAL", ex);
        }
    }

    @Override
    void sendTelemetry(CameraTelemetry telemetry) {
        super.sendTelemetry(telemetry);
        try {
            mgr.sendTelemetry(telemetry);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error sending telemetry to SAL", ex);
        }
    }
    
    @Override
    String getSALVersion() {
        return mgr.getSALVersion();
    }

    @Override
    String getXMLVersion() {
        return mgr.getXMLVersion();
    }
    
    @Override
    String getOSPLVersion() {
        return mgr.getOSPLVersion();
    }
}

==> org-lsst-ccs-subsystem-ocsbridge-2.4.0-sources.jar: org/lsst/ccs/subsystem/ocsbridge/OCSBridgeSALLayer.java <==
package org.lsst.ccs.subsystem.ocsbridge;

import java.time.Duration;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.sal.SAL;
import org.lsst.sal.camera.CameraCommand;
import org.lsst.sal.camera.CameraEvent;
import org.lsst.sal.camera.CameraTelemetry;
import org.lsst.sal.SALException;
import org.lsst.sal.SALReceivedCommand;

/**
 *
 * @author tonyj
 */
public class OCSBridgeSALLayer extends OCSCommandExecutor {

    private final OCSBridge bridge;
    private final SAL<CameraCommand, CameraEvent, CameraTelemetry> mgr;
    private boolean shutdown = false;
    private static final Logger logger = Logger.getLogger(OCSBridgeSALLayer.class.getName());
    private Thread runThread;

    OCSBridgeSALLayer(OCSBridge bridge) {
        super(bridge);
        this.bridge = bridge;
        mgr = bridge.getConfig().getSALManager();
    }

    public void start() {

        Thread t = new Thread("SALCommandReceiver") {

            @Override
            public void run() {
                try {
                    runCommandLoop();
                } catch (SALException x) {
                    logger.log(Level.WARNING, "Failed to initialize SAL communication layer, "
                            + "check that SAL has been setup correctly. Reverting to standalone mode.", x);
                }
            }
        };
        t.start();
    }

    private void runCommandLoop() throws SALException {
        try {
            runThread = Thread.currentThread();

            while (!shutdown) {
                currentCommand = mgr.getNextCommand(Duration.ofMinutes(1));
                if (currentCommand == null) {
                    logger.fine("Still waiting for a command");
                } else {
                    bridge.execute(currentCommand.getCommand());
                }
            }
            //mgr.salShutdown();
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while waiting for commands", ex);
            throw ex;
        } finally {
            runThread = null;
        }
    }
    private SALReceivedCommand<CameraCommand> currentCommand;

    private void shutdown() throws InterruptedException {
        shutdown = true;
        runThread.join();
    }

    @Override
    protected void reportComplete(OCSCommandExecutor.OCSExecutor command) {
        super.reportComplete(command);
        try {
            currentCommand.reportComplete();
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    protected void reportError(OCSCommandExecutor.OCSExecutor command, Exception ex) {
        super.reportError(command, ex);
        try {
            currentCommand.reportError(ex);
        } catch (SALException x) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", x);
        }
    }

    @Override
    protected void acknowledgeCommand(OCSCommandExecutor.OCSExecutor command, Duration timeout) {
        super.acknowledgeCommand(command, timeout);
        try {
            currentCommand.acknowledgeCommand(timeout);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    protected void rejectCommand(OCSCommandExecutor.OCSExecutor command, String reason) {
        super.rejectCommand(command, reason);
        try {
            currentCommand.rejectCommand(reason, 0);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error while responding to command", ex);
        }
    }

    @Override
    void sendEvent(CameraEvent event) {
        super.sendEvent(event);
        try {
            mgr.logEvent(event);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error sending event to SAL", ex);
        }
    }

    @Override
    public void sendTelemetry(CameraTelemetry telemetry) {
        super.sendTelemetry(telemetry);
        try {
            mgr.sendTelemetry(telemetry);
        } catch (SALException ex) {
            logger.log(Level.SEVERE, "Unexpected error sending telemetry to SAL", ex);
        }
    }
    
    @Override
    String getSALVersion() {
        return mgr.getSALVersion();
    }

    @Override
    String getXMLVersion() {
        return mgr.getXMLVersion();
    }
    
    @Override
    String getOSPLVersion() {
        return mgr.getOSPLVersion();
    }
}
